import 'package:flutter/widgets.dart';

///
/// The [Controller] class give a way to update the model and to refresh the view. Also, it provide an easy access to different app controllers.
///
/// Sample class :
/// ```dart
/// class CounterController extends Controller {
///   int _count = 0;
///
///   CounterController({String tag}) : super(tag: tag);
///
///   int get count => _count;
///
///   set count(int v) {
///     _count = v;
///     update();
///   }
///
///   void reset() {
///     _count = 0;
///     update();
///   }
///
///   void increment() {
///     _count++;
///     update();
///   }
/// }
/// ```
///
class Controller extends ChangeNotifier {
  ///
  /// Set of controller objects
  ///
  static final Map _controllers = {};

  ///
  /// Optional controller tag
  ///
  final String tag;

  ///
  /// Constructor
  ///
  Controller({this.tag});

  ///
  /// Find controllers of some type by tag
  /// Method can return null
  ///
  /// Sample use :
  /// To set value 1 for some registred counter tagged 'one':
  /// ```dart
  ///   Controller.find<CounterController>('one')?.single.count = 1;
  /// ```
  ///
  static Iterable<T> find<T extends Controller>(String tag) =>
      of<T>()?.where((element) => element.tag == tag);

  ///
  /// Get all registred controllers of some type
  /// Method can return null
  /// Sample use :
  /// To reset all registred counters:
  /// ```dart
  ///   Controller.of<CounterController>()?.forEach((ctrl) => ctrl.value = 0);
  /// ```
  ///
  static Set<T> of<T extends Controller>() => _controllers[T];

  ///
  /// Register a controller
  ///
  ///Sample use :
  /// To register a counter controller:
  /// ```dart
  ///   Controller.put<CounterController>(CounterController());
  /// ```
  ///
  static T put<T extends Controller>(T controller) {
    if (_controllers[T] == null) _controllers[T] = Set<T>();
    _controllers[T].add(controller);
    return controller;
  }

  ///
  /// Unregister a controller
  ///
  ///Sample use :
  /// To unregister a counter controller:
  /// ```dart
  ///   Controller.remove<CounterController>(ctrl);
  /// ```
  ///
  static bool remove<T extends Controller>(T controller) =>
      _controllers[T].remove(controller);

  ///
  /// Update the view state associated to the controller
  ///
  void update() => notifyListeners();
}
